@@ -77,10 +77,11 @@ func newReconciler(mgr manager.Manager) (reconcile.Reconciler, error) {
 		return nil, err
 	}
 	return &ReconcileChe{
-		client:          mgr.GetClient(),
-		nonCachedClient: noncachedClient,
-		scheme:          mgr.GetScheme(),
-		discoveryClient: discoveryClient,
+		client:            mgr.GetClient(),
+		nonCachedClient:   noncachedClient,
+		scheme:            mgr.GetScheme(),
+		discoveryClient:   discoveryClient,
+		permissionChecker: &K8sApiPermissionChecker{},
 	}, nil
 }
 
@@ -259,8 +260,15 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	return nil
 }
 
-var _ reconcile.Reconciler = &ReconcileChe{}
-var oAuthFinalizerName = "oauthclients.finalizers.che.eclipse.org"
+var (
+	_ reconcile.Reconciler = &ReconcileChe{}
+
+	oAuthFinalizerName                           = "oauthclients.finalizers.che.eclipse.org"
+	cheWorkspacesClusterPermissionsFinalizerName = "cheWorkspaces.clusterpermissions.finalizers.che.eclipse.org"
+
+	// CheServiceAccountName - service account name for che-server.
+	CheServiceAccountName = "che"
+)
 
 // ReconcileChe reconciles a CheCluster object
 type ReconcileChe struct {
@@ -274,9 +282,10 @@ type ReconcileChe struct {
 	nonCachedClient client.Client
 	// A discovery client to check for the existence of certain APIs registered
 	// in the API Server
-	discoveryClient discovery.DiscoveryInterface
-	scheme          *runtime.Scheme
-	tests           bool
+	discoveryClient   discovery.DiscoveryInterface
+	scheme            *runtime.Scheme
+	permissionChecker PermissionChecker
+	tests             bool
 }
 
 const (
@@ -330,6 +339,9 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			return reconcile.Result{}, err
 		}
 	}
+	if r.reconcileWorkspacePermissionsFinalizer(instance, deployContext); err != nil {
+		return reconcile.Result{}, err
+	}
 
 	// Reconcile the imagePuller section of the CheCluster
 	imagePullerResult, err := deploy.ReconcileImagePuller(deployContext)
@@ -461,7 +473,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	}
 
 	// Make sure that CA certificates from all marked config maps are merged into single config map to be propageted to Che components
-	cm, err := deploy.SyncAdditionalCACertsConfigMapToCluster(instance, deployContext)
+	cm, err := deploy.SyncAdditionalCACertsConfigMapToCluster(deployContext)
 	if err != nil {
 		logrus.Errorf("Error updating additional CA config map: %v", err)
 		return reconcile.Result{}, err
@@ -501,7 +513,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 
 	// If the devfile-registry ConfigMap exists, and we are not in airgapped mode, delete the ConfigMap
 	devfileRegistryConfigMap := &corev1.ConfigMap{}
-	err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: instance.Namespace, Name: "devfile-registry"}, devfileRegistryConfigMap)
+	err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: instance.Namespace, Name: deploy.DevfileRegistryName}, devfileRegistryConfigMap)
 	if err != nil && !errors.IsNotFound(err) {
 		logrus.Errorf("Error getting devfile-registry ConfigMap: %v", err)
 		return reconcile.Result{}, err
@@ -517,7 +529,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 
 	// If the plugin-registry ConfigMap exists, and we are not in airgapped mode, delete the ConfigMap
 	pluginRegistryConfigMap := &corev1.ConfigMap{}
-	err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: instance.Namespace, Name: "plugin-registry"}, pluginRegistryConfigMap)
+	err = r.client.Get(context.TODO(), types.NamespacedName{Namespace: instance.Namespace, Name: deploy.PluginRegistryName}, pluginRegistryConfigMap)
 	if err != nil && !errors.IsNotFound(err) {
 		logrus.Errorf("Error getting plugin-registry ConfigMap: %v", err)
 		return reconcile.Result{}, err
@@ -535,10 +547,10 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		return reconcile.Result{}, err
 	}
 
-	// create service accounts:
-	// che is the one which token is used to create workspace objects
-	// che-workspace is SA used by plugins like exec and terminal with limited privileges
-	cheSA, err := deploy.SyncServiceAccountToCluster(deployContext, "che")
+	// Create service account "che" for che-server component.
+	// "che" is the one which token is used to create workspace objects.
+	// Notice: Also we have on more "che-workspace" SA used by plugins like exec, terminal, metrics with limited privileges.
+	cheSA, err := deploy.SyncServiceAccountToCluster(deployContext, CheServiceAccountName)
 	if cheSA == nil {
 		logrus.Info("Waiting on service account 'che' to be created")
 		if err != nil {
@@ -549,49 +561,50 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		}
 	}
 
-	cheWorkspaceSA, err := deploy.SyncServiceAccountToCluster(deployContext, "che-workspace")
-	if cheWorkspaceSA == nil {
-		logrus.Info("Waiting on service account 'che-workspace' to be created")
+	if !util.IsOAuthEnabled(instance) && !util.IsWorkspaceInSameNamespaceWithChe(instance) {
+		clusterRole, err := deploy.GetClusterRole(CheWorkspacesClusterRoleNameTemplate, deployContext.ClusterAPI.Client)
 		if err != nil {
 			logrus.Error(err)
-		}
-		if !tests {
 			return reconcile.Result{RequeueAfter: time.Second}, err
 		}
+		if clusterRole == nil {
+			policies := append(getCheWorkspacesNamespacePolicy(), getCheWorkspacesPolicy()...)
+			deniedRules, err := r.permissionChecker.GetNotPermittedPolicyRules(policies, "")
+			if err != nil {
+				logrus.Error(err)
+				return reconcile.Result{RequeueAfter: time.Second}, err
+			}
+			// fall back to the "narrower" workspace namespace strategy
+			if len(deniedRules) > 0 {
+				logrus.Warnf("Not enough permissions to start a workspace in dedicated namespace. Denied policies: %v", deniedRules)
+				logrus.Warnf("Fall back to '%s' namespace for workspaces.", instance.Namespace)
+				delete(instance.Spec.Server.CustomCheProperties, "CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT")
+				instance.Spec.Server.WorkspaceNamespaceDefault = instance.Namespace
+				r.UpdateCheCRSpec(instance, "Default namespace for workspaces", instance.Namespace);
+				if err != nil {
+					logrus.Error(err)
+					return reconcile.Result{RequeueAfter: time.Second * 1}, err
+				}
+			} else {
+				reconcileResult, err := r.delegateWorkspacePermissionsInTheDifferNamespaceThanChe(instance, deployContext)
+				if err != nil {
+					logrus.Error(err)
+					return reconcile.Result{RequeueAfter: time.Second}, err
+				}
+				if reconcileResult.Requeue {
+					return reconcileResult, err
+				}
+			}
+		}
 	}
 
-	// create exec role for CheCluster server and workspaces
-	execRole, err := deploy.SyncExecRoleToCluster(deployContext)
-	if execRole == nil {
-		logrus.Info("Waiting on role 'exec' to be created")
+	if util.IsOAuthEnabled(instance) || util.IsWorkspaceInSameNamespaceWithChe(instance) {
+		reconcile, err := r.delegateWorkspacePermissionsInTheSameNamespaceWithChe(deployContext)
 		if err != nil {
 			logrus.Error(err)
 		}
-		if !tests {
-			return reconcile.Result{RequeueAfter: time.Second}, err
-		}
-	}
-
-	// create view role for CheCluster server and workspaces
-	viewRole, err := deploy.SyncViewRoleToCluster(deployContext)
-	if viewRole == nil {
-		logrus.Info("Waiting on role 'view' to be created")
-		if err != nil {
-			logrus.Error(err)
-		}
-		if !tests {
-			return reconcile.Result{RequeueAfter: time.Second}, err
-		}
-	}
-
-	cheRoleBinding, err := deploy.SyncRoleBindingToCluster(deployContext, "che", "che", "edit", "ClusterRole")
-	if cheRoleBinding == nil {
-		logrus.Info("Waiting on role binding 'che' to be created")
-		if err != nil {
-			logrus.Error(err)
-		}
-		if !tests {
-			return reconcile.Result{RequeueAfter: time.Second}, err
+		if reconcile.Requeue && !tests {
+			return reconcile, err
 		}
 	}
 
@@ -613,28 +626,6 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		}
 	}
 
-	cheWSExecRoleBinding, err := deploy.SyncRoleBindingToCluster(deployContext, "che-workspace-exec", "che-workspace", "exec", "Role")
-	if cheWSExecRoleBinding == nil {
-		logrus.Info("Waiting on role binding 'che-workspace-exec' to be created")
-		if err != nil {
-			logrus.Error(err)
-		}
-		if !tests {
-			return reconcile.Result{RequeueAfter: time.Second}, err
-		}
-	}
-
-	cheWSViewRoleBinding, err := deploy.SyncRoleBindingToCluster(deployContext, "che-workspace-view", "che-workspace", "view", "Role")
-	if cheWSViewRoleBinding == nil {
-		logrus.Info("Waiting on role binding 'che-workspace-view' to be created")
-		if err != nil {
-			logrus.Error(err)
-		}
-		if !tests {
-			return reconcile.Result{RequeueAfter: time.Second}, err
-		}
-	}
-
 	// If the user specified an additional cluster role to use for the Che workspace, create a role binding for it
 	// Use a role binding instead of a cluster role binding to keep the additional access scoped to the workspace's namespace
 	workspaceClusterRole := instance.Spec.Server.CheWorkspaceClusterRole
@@ -658,8 +649,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	cheMultiUser := deploy.GetCheMultiUser(instance)
 
 	if cheMultiUser == "false" {
-		labels := deploy.GetLabels(instance, cheFlavor)
-		pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultCheVolumeClaimName, "1Gi", labels)
+		pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultCheVolumeClaimName, "1Gi", cheFlavor)
 		if !tests {
 			if !pvcStatus.Continue {
 				logrus.Infof("Waiting on pvc '%s' to be bound. Sometimes PVC can be bound only when the first consumer is created.", deploy.DefaultCheVolumeClaimName)
@@ -685,14 +675,12 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	externalDB := instance.Spec.Database.ExternalDb
 	if !externalDB {
 		if cheMultiUser == "false" {
-			if util.K8sclient.IsDeploymentExists(postgres.PostgresDeploymentName, instance.Namespace) {
-				util.K8sclient.DeleteDeployment(postgres.PostgresDeploymentName, instance.Namespace)
+			if util.K8sclient.IsDeploymentExists(deploy.PostgresName, instance.Namespace) {
+				util.K8sclient.DeleteDeployment(deploy.PostgresName, instance.Namespace)
 			}
 		} else {
-			postgresLabels := deploy.GetLabels(instance, postgres.PostgresDeploymentName)
-
 			// Create a new postgres service
-			serviceStatus := deploy.SyncServiceToCluster(deployContext, "postgres", []string{"postgres"}, []int32{5432}, postgresLabels)
+			serviceStatus := deploy.SyncServiceToCluster(deployContext, deploy.PostgresName, []string{deploy.PostgresName}, []int32{5432}, deploy.PostgresName)
 			if !tests {
 				if !serviceStatus.Continue {
 					logrus.Info("Waiting on service 'postgres' to be ready")
@@ -705,7 +693,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			}
 
 			// Create a new Postgres PVC object
-			pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultPostgresVolumeClaimName, "1Gi", postgresLabels)
+			pvcStatus := deploy.SyncPVCToCluster(deployContext, deploy.DefaultPostgresVolumeClaimName, "1Gi", deploy.PostgresName)
 			if !tests {
 				if !pvcStatus.Continue {
 					logrus.Infof("Waiting on pvc '%s' to be bound. Sometimes PVC can be bound only when the first consumer is created.", deploy.DefaultPostgresVolumeClaimName)
@@ -718,15 +706,15 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			}
 
 			// Create a new Postgres deployment
-			deploymentStatus := postgres.SyncPostgresDeploymentToCluster(deployContext)
+			provisioned, err := postgres.SyncPostgresDeploymentToCluster(deployContext)
 			if !tests {
-				if !deploymentStatus.Continue {
-					logrus.Infof("Waiting on deployment '%s' to be ready", postgres.PostgresDeploymentName)
-					if deploymentStatus.Err != nil {
-						logrus.Error(deploymentStatus.Err)
+				if !provisioned {
+					logrus.Infof("Waiting on deployment '%s' to be ready", deploy.PostgresName)
+					if err != nil {
+						logrus.Error(err)
 					}
 
-					return reconcile.Result{Requeue: deploymentStatus.Requeue}, deploymentStatus.Err
+					return reconcile.Result{}, err
 				}
 			}
 
@@ -741,15 +729,16 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 					}
 					identityProviderPostgresPassword = password
 				}
-				pgCommand := identity_provider.GetPostgresProvisionCommand(identityProviderPostgresPassword)
 				dbStatus := instance.Status.DbProvisoned
 				// provision Db and users for Che and Keycloak servers
 				if !dbStatus {
-					podToExec, err := util.K8sclient.GetDeploymentPod(postgres.PostgresDeploymentName, instance.Namespace)
-					if err != nil {
-						return reconcile.Result{}, err
-					}
-					_, err = util.K8sclient.ExecIntoPod(podToExec, pgCommand, "create Keycloak DB, user, privileges", instance.Namespace)
+					_, err := util.K8sclient.ExecIntoPod(
+						instance,
+						deploy.PostgresName,
+						func(cr *orgv1.CheCluster) (string, error) {
+							return identity_provider.GetPostgresProvisionCommand(identityProviderPostgresPassword), nil
+						},
+						"create Keycloak DB, user, privileges")
 					if err == nil {
 						for {
 							instance.Status.DbProvisoned = true
@@ -793,7 +782,14 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	cheHost := ""
 	if !isOpenShift {
 		additionalLabels := deployContext.CheCluster.Spec.Server.CheServerIngress.Labels
-		ingress, err := deploy.SyncIngressToCluster(deployContext, cheFlavor, instance.Spec.Server.CheHost, exposedServiceName, 8080, additionalLabels)
+		ingress, err := deploy.SyncIngressToCluster(
+			deployContext,
+			cheFlavor,
+			instance.Spec.Server.CheHost,
+			exposedServiceName,
+			8080,
+			additionalLabels,
+			cheFlavor)
 		if !tests {
 			if ingress == nil {
 				logrus.Infof("Waiting on ingress '%s' to be ready", cheFlavor)
@@ -813,7 +809,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 		}
 
 		additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
-		route, err := deploy.SyncRouteToCluster(deployContext, cheFlavor, customHost, exposedServiceName, 8080, additionalLabels)
+		route, err := deploy.SyncRouteToCluster(deployContext, cheFlavor, customHost, exposedServiceName, 8080, additionalLabels, cheFlavor)
 		if route == nil {
 			logrus.Infof("Waiting on route '%s' to be ready", cheFlavor)
 			if err != nil {
@@ -842,7 +838,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 			if err != nil {
 				logrus.Errorf("Error provisioning the identity provider to cluster: %v", err)
 			}
-			return reconcile.Result{RequeueAfter: time.Second * 1}, err
+			return reconcile.Result{}, err
 		}
 	}
 
@@ -850,7 +846,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	if !tests {
 		if !provisioned {
 			if err != nil {
-				logrus.Errorf("Error provisioning '%s' to cluster: %v", deploy.DevfileRegistry, err)
+				logrus.Errorf("Error provisioning '%s' to cluster: %v", deploy.DevfileRegistryName, err)
 			}
 			return reconcile.Result{RequeueAfter: time.Second * 1}, err
 		}
@@ -860,7 +856,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	if !tests {
 		if !provisioned {
 			if err != nil {
-				logrus.Errorf("Error provisioning '%s' to cluster: %v", deploy.PluginRegistry, err)
+				logrus.Errorf("Error provisioning '%s' to cluster: %v", deploy.PluginRegistryName, err)
 			}
 			return reconcile.Result{RequeueAfter: time.Second * 1}, err
 		}
@@ -886,12 +882,12 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	}
 
 	// Create a new che deployment
-	deploymentStatus := server.SyncCheDeploymentToCluster(deployContext)
+	provisioned, err = server.SyncCheDeploymentToCluster(deployContext)
 	if !tests {
-		if !deploymentStatus.Continue {
+		if !provisioned {
 			logrus.Infof("Waiting on deployment '%s' to be ready", cheFlavor)
-			if deploymentStatus.Err != nil {
-				logrus.Error(deploymentStatus.Err)
+			if err != nil {
+				logrus.Error(err)
 			}
 
 			deployment, err := r.GetEffectiveDeployment(instance, cheFlavor)
@@ -912,7 +908,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 					}
 				}
 			}
-			return reconcile.Result{Requeue: deploymentStatus.Requeue}, deploymentStatus.Err
+			return reconcile.Result{}, err
 		}
 	}
 	// Update available status
@@ -945,7 +941,7 @@ func (r *ReconcileChe) Reconcile(request reconcile.Request) (reconcile.Result, e
 	deleted, err := r.ReconcileIdentityProvider(instance, isOpenShift4)
 	if deleted {
 		for {
-			if err := r.DeleteFinalizer(instance); err != nil &&
+			if err := r.DeleteOAuthFinalizer(instance); err != nil &&
 				errors.IsConflict(err) {
 				instance, _ = r.GetCR(request)
 				continue
@@ -992,6 +988,9 @@ func createConsoleLink(isOpenShift4 bool, protocol string, instance *orgv1.CheCl
 	preparedConsoleLink := &consolev1.ConsoleLink{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: deploy.DefaultConsoleLinkName(),
+			Annotations: map[string]string{
+				deploy.CheEclipseOrgNamespace: instance.Namespace,
+			},
 		},
 		Spec: consolev1.ConsoleLinkSpec{
 			Link: consolev1.Link{
@@ -1046,11 +1045,11 @@ func EvaluateCheServerVersion(cr *orgv1.CheCluster) string {
 }
 
 func getDefaultCheHost(deployContext *deploy.DeployContext) (string, error) {
-	routeName := deploy.DefaultCheFlavor(deployContext.CheCluster)
+	cheFlavor := deploy.DefaultCheFlavor(deployContext.CheCluster)
 	additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
-	route, err := deploy.SyncRouteToCluster(deployContext, routeName, "", getServerExposingServiceName(deployContext.CheCluster), 8080, additionalLabels)
+	route, err := deploy.SyncRouteToCluster(deployContext, cheFlavor, "", getServerExposingServiceName(deployContext.CheCluster), 8080, additionalLabels, cheFlavor)
 	if route == nil {
-		logrus.Infof("Waiting on route '%s' to be ready", routeName)
+		logrus.Infof("Waiting on route '%s' to be ready", cheFlavor)
 		if err != nil {
 			logrus.Error(err)
 		}
