@@ -17,11 +17,14 @@ import (
 	"io/ioutil"
 	"os"
 
+	mocks "github.com/eclipse/che-operator/mocks"
+
 	"reflect"
 	"time"
 
 	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
 	identity_provider "github.com/eclipse/che-operator/pkg/deploy/identity-provider"
+	"github.com/golang/mock/gomock"
 	"github.com/google/go-cmp/cmp"
 
 	"github.com/eclipse/che-operator/pkg/deploy"
@@ -39,11 +42,10 @@ import (
 	packagesv1 "github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/apis/operators/v1"
 
 	"github.com/sirupsen/logrus"
-	"gopkg.in/yaml.v2"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
+	rbac "k8s.io/api/rbac/v1"
 
-	rbacapi "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -58,6 +60,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
+	"sigs.k8s.io/yaml"
 
 	"testing"
 )
@@ -185,6 +188,7 @@ var (
 			},
 		},
 	}
+	route = &routev1.Route{}
 )
 
 func init() {
@@ -340,6 +344,7 @@ func TestCaseAutoDetectOAuth(t *testing.T) {
 			scheme.AddKnownTypes(oauth.SchemeGroupVersion, oAuthClient)
 			scheme.AddKnownTypes(userv1.SchemeGroupVersion, &userv1.UserList{}, &userv1.User{})
 			scheme.AddKnownTypes(oauth_config.SchemeGroupVersion, &oauth_config.OAuth{})
+			scheme.AddKnownTypes(routev1.GroupVersion, route)
 
 			initCR := InitCheWithSimpleCR()
 			initCR.Spec.Auth.OpenShiftoAuth = testCase.initialOAuthValue
@@ -360,6 +365,7 @@ func TestCaseAutoDetectOAuth(t *testing.T) {
 				nonCachedClient: nonCachedClient,
 				discoveryClient: fakeDiscovery,
 				scheme:          scheme,
+				tests:           true,
 			}
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
@@ -541,6 +547,7 @@ func TestImagePullerConfiguration(t *testing.T) {
 			operatorsv1alpha1.AddToScheme(scheme.Scheme)
 			operatorsv1.AddToScheme(scheme.Scheme)
 			chev1alpha1.AddToScheme(scheme.Scheme)
+			routev1.AddToScheme(scheme.Scheme)
 			testCase.initObjects = append(testCase.initObjects, testCase.initCR)
 			cli := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
 			nonCachedClient := fake.NewFakeClientWithScheme(scheme.Scheme, testCase.initObjects...)
@@ -580,6 +587,7 @@ func TestImagePullerConfiguration(t *testing.T) {
 				nonCachedClient: nonCachedClient,
 				discoveryClient: fakeDiscovery,
 				scheme:          scheme.Scheme,
+				tests:           true,
 			}
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
@@ -707,7 +715,7 @@ func TestCheController(t *testing.T) {
 
 	// get devfile-registry configmap
 	devfilecm := &corev1.ConfigMap{}
-	if err := cl.Get(context.TODO(), types.NamespacedName{Name: "devfile-registry", Namespace: cheCR.Namespace}, devfilecm); err != nil {
+	if err := cl.Get(context.TODO(), types.NamespacedName{Name: deploy.DevfileRegistryName, Namespace: cheCR.Namespace}, devfilecm); err != nil {
 		t.Errorf("ConfigMap %s not found: %s", devfilecm.Name, err)
 	}
 
@@ -746,7 +754,7 @@ func TestCheController(t *testing.T) {
 	}
 
 	// Get the custom role binding that should have been created for the role we passed in
-	rb := &rbacapi.RoleBinding{}
+	rb := &rbac.RoleBinding{}
 	if err := cl.Get(context.TODO(), types.NamespacedName{Name: "che-workspace-custom", Namespace: cheCR.Namespace}, rb); err != nil {
 		t.Errorf("Custom role binding %s not found: %s", rb.Name, err)
 	}
@@ -809,7 +817,7 @@ func TestCheController(t *testing.T) {
 	if err = r.client.Get(context.TODO(), types.NamespacedName{Name: cheCR.Name, Namespace: cheCR.Namespace}, cheCR); err != nil {
 		t.Errorf("Failed to get the Che custom resource %s: %s", cheCR.Name, err)
 	}
-	if err = identity_provider.SyncIdentityProviderItems(deployContext, "che"); err != nil {
+	if _, err = identity_provider.SyncOpenShiftIdentityProviderItems(deployContext); err != nil {
 		t.Errorf("Failed to create the items for the identity provider: %s", err)
 	}
 	oAuthClientName := cheCR.Spec.Auth.OAuthClientName
@@ -828,13 +836,13 @@ func TestCheController(t *testing.T) {
 		t.Error("Failed to update CheCluster custom resource")
 	}
 	postgresDeployment := &appsv1.Deployment{}
-	err = r.client.Get(context.TODO(), types.NamespacedName{Name: "postgres", Namespace: cheCR.Namespace}, postgresDeployment)
+	err = r.client.Get(context.TODO(), types.NamespacedName{Name: deploy.PostgresName, Namespace: cheCR.Namespace}, postgresDeployment)
 	err = r.client.Delete(context.TODO(), postgresDeployment)
 	_, err = r.Reconcile(req)
 	if err != nil {
 		t.Fatalf("reconcile: (%v)", err)
 	}
-	err = r.client.Get(context.TODO(), types.NamespacedName{Name: "postgres", Namespace: cheCR.Namespace}, postgresDeployment)
+	err = r.client.Get(context.TODO(), types.NamespacedName{Name: deploy.PostgresName, Namespace: cheCR.Namespace}, postgresDeployment)
 	if err == nil {
 		t.Fatalf("Deployment postgres shoud not exist")
 	}
@@ -938,139 +946,129 @@ func TestConfiguringLabelsForRoutes(t *testing.T) {
 	}
 }
 
-func TestShouldSetUpCorrectlyInternalIdentityProviderServiceURL(t *testing.T) {
+func TestShouldDelegatePermissionsForCheWorkspaces(t *testing.T) {
 	os.Setenv("OPENSHIFT_VERSION", "3")
-
 	type testCase struct {
-		name              string
-		cheCR *orgv1.CheCluster
-		expectedIdentityProviderInternalURL string
+		name        string
+		initObjects []runtime.Object
+
+		clusterRole bool
+		checluster  *orgv1.CheCluster
 	}
 
-	// Set the logger to development mode for verbose logs.
-	logf.SetLogger(logf.ZapLogger(true))
+	// the same namespace with Che
+	crWsInTheSameNs1 := InitCheWithSimpleCR().DeepCopy()
+	crWsInTheSameNs1.Spec.Server.WorkspaceNamespaceDefault = crWsInTheSameNs1.Namespace
+
+	crWsInTheSameNs2 := InitCheWithSimpleCR().DeepCopy()
+	crWsInTheSameNs2.Spec.Server.WorkspaceNamespaceDefault = ""
+
+	crWsInTheSameNs3 := InitCheWithSimpleCR().DeepCopy()
+	crWsInTheSameNs3.Spec.Server.CustomCheProperties = make(map[string]string)
+	crWsInTheSameNs3.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"] = ""
+
+	crWsInTheSameNs4 := InitCheWithSimpleCR().DeepCopy()
+	crWsInTheSameNs4.Spec.Server.CustomCheProperties = make(map[string]string)
+	crWsInTheSameNs4.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"] = crWsInTheSameNs1.Namespace
+
+	// differ namespace with Che
+	crWsInAnotherNs1 := InitCheWithSimpleCR().DeepCopy()
+	crWsInAnotherNs1.Spec.Server.WorkspaceNamespaceDefault = "some-test-namespace"
+
+	crWsInAnotherNs2 := InitCheWithSimpleCR().DeepCopy()
+	crWsInAnotherNs2.Spec.Server.CustomCheProperties = make(map[string]string)
+	crWsInAnotherNs2.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"] = "some-test-namespace"
+
+	crWsInAnotherNs3 := InitCheWithSimpleCR().DeepCopy()
+	crWsInAnotherNs3.Spec.Server.CustomCheProperties = make(map[string]string)
+	crWsInAnotherNs3.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"] = crWsInTheSameNs1.Namespace
+	crWsInAnotherNs3.Spec.Server.WorkspaceNamespaceDefault = "some-test-namespace"
 
 	testCases := []testCase{
 		{
-			name: "Should use 'external' public identity provider url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-						ExternalIdentityProvider: true,
-						IdentityProviderURL: "http://external-keycloak",
-					},
-				},
-			},
-			expectedIdentityProviderInternalURL: "http://external-keycloak/auth",
+			name:        "che-operator should delegate permission for workspaces in the same namespace with Che. WorkspaceNamespaceDefault=" + crWsInTheSameNs1.Namespace,
+			initObjects: []runtime.Object{},
+			clusterRole: false,
+			checluster:  crWsInTheSameNs1,
 		},
 		{
-			name: "Should use 'external' public identity provider url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-						ExternalIdentityProvider: true,
-						IdentityProviderURL: "http://external-keycloak",
-					},
-				},
-			},
-			expectedIdentityProviderInternalURL: "http://external-keycloak/auth",
+			name:        "che-operator should delegate permission for workspaces in the same namespace with Che. WorkspaceNamespaceDefault=''",
+			initObjects: []runtime.Object{},
+			clusterRole: false,
+			checluster:  crWsInTheSameNs2,
 		},
 		{
-			name: "Should use public identity provider url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-						ExternalIdentityProvider: false,
-					},
-				},
-			},
-			expectedIdentityProviderInternalURL: "http://keycloak/auth",
+			name:        "che-operator should delegate permission for workspaces in the same namespace with Che. Property CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT=''",
+			initObjects: []runtime.Object{},
+			clusterRole: false,
+			checluster:  crWsInTheSameNs3,
 		},
 		{
-			name: "Should use internal identity provider url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-						ExternalIdentityProvider: false,
-					},
-				},
-			},
-			expectedIdentityProviderInternalURL: "http://keycloak.eclipse-che.svc:8080/auth",
+			name:        "che-operator should delegate permission for workspaces in the same namespace with Che. Property CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT=" + crWsInTheSameNs1.Namespace,
+			initObjects: []runtime.Object{},
+			clusterRole: false,
+			checluster:  crWsInTheSameNs4,
+		},
+		{
+			name:        "che-operator should delegate permission for workspaces in differ namespace than Che. WorkspaceNamespaceDefault = 'some-test-namespace'",
+			initObjects: []runtime.Object{},
+			clusterRole: true,
+			checluster:  crWsInAnotherNs1,
+		},
+		{
+			name:        "che-operator should delegate permission for workspaces in differ namespace than Che. Property CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT = 'some-test-namespace'",
+			initObjects: []runtime.Object{},
+			clusterRole: true,
+			checluster:  crWsInAnotherNs2,
+		},
+		{
+			name:        "che-operator should delegate permission for workspaces in differ namespace than Che. Property CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT points to Che namespace with higher priority WorkspaceNamespaceDefault = 'some-test-namespace'.",
+			initObjects: []runtime.Object{},
+			clusterRole: false,
+			checluster:  crWsInAnotherNs3,
 		},
 	}
-
 	for _, testCase := range testCases {
 		t.Run(testCase.name, func(t *testing.T) {
+			logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+
 			scheme := scheme.Scheme
 			orgv1.SchemeBuilder.AddToScheme(scheme)
-			scheme.AddKnownTypes(routev1.SchemeGroupVersion, &routev1.Route{})
+			scheme.AddKnownTypes(oauth.SchemeGroupVersion, oAuthClient)
+			scheme.AddKnownTypes(userv1.SchemeGroupVersion, &userv1.UserList{}, &userv1.User{})
+			scheme.AddKnownTypes(oauth_config.SchemeGroupVersion, &oauth_config.OAuth{})
+			scheme.AddKnownTypes(routev1.GroupVersion, route)
 
-			cli := fake.NewFakeClientWithScheme(scheme, testCase.cheCR, )
-			nonCachedClient := fake.NewFakeClientWithScheme(scheme, testCase.cheCR)
+			initCR := testCase.checluster
+			initCR.Spec.Auth.OpenShiftoAuth = util.NewBoolPointer(false)
+			testCase.initObjects = append(testCase.initObjects, initCR)
+
+			cli := fake.NewFakeClientWithScheme(scheme, testCase.initObjects...)
+			nonCachedClient := fake.NewFakeClientWithScheme(scheme, testCase.initObjects...)
 			clientSet := fakeclientset.NewSimpleClientset()
+			// todo do we need fake discovery
 			fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
 			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
+
 			if !ok {
 				t.Fatal("Error creating fake discovery client")
 			}
+
+			var m *mocks.MockPermissionChecker
+			if testCase.clusterRole {
+				ctrl := gomock.NewController(t)
+				m = mocks.NewMockPermissionChecker(ctrl)
+				m.EXPECT().GetNotPermittedPolicyRules(gomock.Any(), "").Return([]rbac.PolicyRule{}, nil).MaxTimes(2)
+				defer ctrl.Finish()
+			}
+
 			r := &ReconcileChe{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				scheme:          scheme,
-				tests: true,
+				client:            cli,
+				nonCachedClient:   nonCachedClient,
+				discoveryClient:   fakeDiscovery,
+				scheme:            scheme,
+				permissionChecker: m,
+				tests:             true,
 			}
 			req := reconcile.Request{
 				NamespacedName: types.NamespacedName{
@@ -1079,22 +1077,6 @@ func TestShouldSetUpCorrectlyInternalIdentityProviderServiceURL(t *testing.T) {
 				},
 			}
 
-			clusterAPI := deploy.ClusterAPI{
-				Client: r.client,
-				Scheme: r.scheme,
-			}
-			deployContext := &deploy.DeployContext{
-				CheCluster: testCase.cheCR,
-				ClusterAPI: clusterAPI,
-			}
-
-			// Set up che host for route
-			cheRoute, _ := deploy.GetSpecRoute(deployContext, deploy.DefaultCheFlavor(testCase.cheCR), "che-host", "che-host", 8080, "")
-			r.client.Create(context.TODO(), cheRoute)
-			// Set up keycloak host for route
-			keycloakRoute, _ := deploy.GetSpecRoute(deployContext, "keycloak", "keycloak", "keycloak", 8080, "")
-			r.client.Create(context.TODO(), keycloakRoute)
-
 			_, err := r.Reconcile(req)
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
@@ -1103,530 +1085,124 @@ func TestShouldSetUpCorrectlyInternalIdentityProviderServiceURL(t *testing.T) {
 			if err != nil {
 				t.Fatalf("Error reconciling: %v", err)
 			}
-			_, err = r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
 
-			cheCm := &corev1.ConfigMap{}
-			if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "che", Namespace: testCase.cheCR.Namespace}, cheCm); err != nil {
-				t.Errorf("ConfigMap %s not found: %s", cheCm.Name, err)
-			}
+			if !testCase.clusterRole {
+				viewRole := &rbac.Role{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: deploy.ViewRoleName, Namespace: namespace}, viewRole); err != nil {
+					t.Errorf("role '%s' not found", deploy.ViewRoleName)
+				}
+				viewRoleBinding := &rbac.RoleBinding{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: ViewRoleBindingName, Namespace: namespace}, viewRoleBinding); err != nil {
+					t.Errorf("rolebinding '%s' not found", ViewRoleBindingName)
+				}
 
-			keycloakInternalURLActual := cheCm.Data["CHE_KEYCLOAK_AUTH__INTERNAL__SERVER__URL"]
-			keycloakInternalURLExpected := testCase.expectedIdentityProviderInternalURL
-			if keycloakInternalURLActual != keycloakInternalURLExpected {
-				t.Fatalf("Keycloak internal url must be %s", keycloakInternalURLExpected)
+				execRole := &rbac.Role{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: deploy.ExecRoleName, Namespace: namespace}, execRole); err != nil {
+					t.Errorf("role '%s' not found", deploy.ExecRoleName)
+				}
+				execRoleBinding := &rbac.RoleBinding{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: ExecRoleBindingName, Namespace: namespace}, execRoleBinding); err != nil {
+					t.Errorf("rolebinding '%s' not found", ExecRoleBindingName)
+				}
+
+				editRoleBinding := &rbac.RoleBinding{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: EditRoleBindingName, Namespace: namespace}, editRoleBinding); err != nil {
+					t.Errorf("rolebinding '%s' not found", EditRoleBindingName)
+				}
+			} else {
+				manageNamespacesClusterRoleName := fmt.Sprintf(CheWorkspacesNamespaceClusterRoleNameTemplate, namespace)
+				cheManageNamespaceClusterRole := &rbac.ClusterRole{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: manageNamespacesClusterRoleName}, cheManageNamespaceClusterRole); err != nil {
+					t.Errorf("role '%s' not found", manageNamespacesClusterRoleName)
+				}
+				cheManageNamespaceClusterRoleBinding := &rbac.ClusterRoleBinding{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: manageNamespacesClusterRoleName}, cheManageNamespaceClusterRoleBinding); err != nil {
+					t.Errorf("rolebinding '%s' not found", manageNamespacesClusterRoleName)
+				}
+
+				cheWorkspacesClusterRoleName := fmt.Sprintf(CheWorkspacesClusterRoleNameTemplate, namespace)
+				cheWorkspacesClusterRole := &rbac.ClusterRole{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: cheWorkspacesClusterRoleName}, cheWorkspacesClusterRole); err != nil {
+					t.Errorf("role '%s' not found", cheWorkspacesClusterRole)
+				}
+				cheWorkspacesClusterRoleBinding := &rbac.ClusterRoleBinding{}
+				if err := r.client.Get(context.TODO(), types.NamespacedName{Name: cheWorkspacesClusterRoleName}, cheWorkspacesClusterRoleBinding); err != nil {
+					t.Errorf("rolebinding '%s' not found", cheWorkspacesClusterRole)
+				}
 			}
 		})
 	}
 }
 
-func TestShouldUsePublicUrlForExternalPluginRegistryWhenInternalNetworkEnabled(t *testing.T) {
-	os.Setenv("OPENSHIFT_VERSION", "3")
+func TestShouldFallBackWorspaceNamespaceDefaultBecauseNotEnoughtPermissions(t *testing.T) {
+	// the same namespace with Che
+	cr := InitCheWithSimpleCR().DeepCopy()
+	cr.Spec.Server.WorkspaceNamespaceDefault = "che-workspace-<username>"
 
-	type testCase struct {
-		name              string
-		cheCR *orgv1.CheCluster
-		expectedPluginRegistryInternalURL string
+	logf.SetLogger(zap.LoggerTo(os.Stdout, true))
+
+	scheme := scheme.Scheme
+	orgv1.SchemeBuilder.AddToScheme(scheme)
+	scheme.AddKnownTypes(oauth.SchemeGroupVersion, oAuthClient)
+	scheme.AddKnownTypes(userv1.SchemeGroupVersion, &userv1.UserList{}, &userv1.User{})
+	scheme.AddKnownTypes(oauth_config.SchemeGroupVersion, &oauth_config.OAuth{})
+	scheme.AddKnownTypes(routev1.GroupVersion, route)
+
+	cr.Spec.Auth.OpenShiftoAuth = util.NewBoolPointer(false)
+
+	cli := fake.NewFakeClientWithScheme(scheme, cr)
+	nonCachedClient := fake.NewFakeClientWithScheme(scheme, cr)
+	clientSet := fakeclientset.NewSimpleClientset()
+	// todo do we need fake discovery
+	fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
+	fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
+
+	if !ok {
+		t.Fatal("Error creating fake discovery client")
 	}
 
-	// Set the logger to development mode for verbose logs.
-	logf.SetLogger(logf.ZapLogger(true))
+	var m *mocks.MockPermissionChecker
+	ctrl := gomock.NewController(t)
+	m = mocks.NewMockPermissionChecker(ctrl)
+	m.EXPECT().GetNotPermittedPolicyRules(gomock.Any(), "").Return([]rbac.PolicyRule{
+		{
+			APIGroups: []string{""},
+			Resources: []string{"namespaces"},
+			Verbs:     []string{"get", "create", "update"},
+		},
+	}, nil).MaxTimes(2)
+	defer ctrl.Finish()
 
-	testCases := []testCase{
-		{
-			name: "Should use 'external' public plugin registry url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-						ExternalPluginRegistry: true,
-						PluginRegistryUrl: "http://external-plugin-registry",
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedPluginRegistryInternalURL: "http://external-plugin-registry",
-		},
-		{
-			name: "Should use 'external' public plugin registry url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-						ExternalPluginRegistry: true,
-						PluginRegistryUrl: "http://external-plugin-registry",
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedPluginRegistryInternalURL: "http://external-plugin-registry",
-		},
-		{
-			name: "Should use public plugin registry url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-						ExternalPluginRegistry: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedPluginRegistryInternalURL: "http://plugin-registry/v3",
-		},
-		{
-			name: "Should use internal plugin registry url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-						ExternalPluginRegistry: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedPluginRegistryInternalURL: "http://plugin-registry.eclipse-che.svc:8080/v3",
+	r := &ReconcileChe{
+		client:            cli,
+		nonCachedClient:   nonCachedClient,
+		discoveryClient:   fakeDiscovery,
+		scheme:            scheme,
+		permissionChecker: m,
+		tests:             true,
+	}
+	req := reconcile.Request{
+		NamespacedName: types.NamespacedName{
+			Name:      name,
+			Namespace: namespace,
 		},
 	}
 
-	for _, testCase := range testCases {
-		t.Run(testCase.name, func(t *testing.T) {
-			scheme := scheme.Scheme
-			orgv1.SchemeBuilder.AddToScheme(scheme)
-			scheme.AddKnownTypes(routev1.SchemeGroupVersion, &routev1.Route{})
-
-			cli := fake.NewFakeClientWithScheme(scheme, testCase.cheCR, )
-			nonCachedClient := fake.NewFakeClientWithScheme(scheme, testCase.cheCR)
-			clientSet := fakeclientset.NewSimpleClientset()
-			fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
-			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
-			if !ok {
-				t.Fatal("Error creating fake discovery client")
-			}
-			r := &ReconcileChe{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				scheme:          scheme,
-				tests: true,
-			}
-			req := reconcile.Request{
-				NamespacedName: types.NamespacedName{
-					Name:      name,
-					Namespace: namespace,
-				},
-			}
-
-			clusterAPI := deploy.ClusterAPI{
-				Client: r.client,
-				Scheme: r.scheme,
-			}
-			deployContext := &deploy.DeployContext{
-				CheCluster: testCase.cheCR,
-				ClusterAPI: clusterAPI,
-			}
-
-			// Set up che host for route
-			cheRoute, _ := deploy.GetSpecRoute(deployContext, deploy.DefaultCheFlavor(testCase.cheCR), "che-host", "che-host", 8080, "")
-			r.client.Create(context.TODO(), cheRoute);
-			// Set up keycloak host for route
-			keycloakRoute, _ := deploy.GetSpecRoute(deployContext, "keycloak", "keycloak", "keycloak", 8080, "")
-			r.client.Create(context.TODO(), keycloakRoute)
-			// Set up plugin registry host for route
-			pluginRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "plugin-registry", "plugin-registry", "plugin-registry", 8080, "")
-			r.client.Create(context.TODO(), pluginRegistryRoute)
-			// Set up devfile registry host for route
-			devfileRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "devfile-registry", "devfile-registry", "devfile-registry", 8080, "")
-			r.client.Create(context.TODO(), devfileRegistryRoute)
-			
-
-			_, err := r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-			_, err = r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-
-			cheCm := &corev1.ConfigMap{}
-			if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "che", Namespace: testCase.cheCR.Namespace}, cheCm); err != nil {
-				t.Errorf("ConfigMap %s not found: %s", cheCm.Name, err)
-			}
-
-			pluginRegistryInternalURLActual := cheCm.Data["CHE_WORKSPACE_PLUGIN__REGISTRY__INTERNAL__URL"]
-			pluginRegistryInternalURLExpected := testCase.expectedPluginRegistryInternalURL
-			if pluginRegistryInternalURLActual != pluginRegistryInternalURLExpected {
-				t.Fatalf("plugin registry internal url must be %s", pluginRegistryInternalURLExpected)
-			}
-		})
+	_, err := r.Reconcile(req)
+	if err != nil {
+		t.Fatalf("Error reconciling: %v", err)
 	}
-}
-
-func TestShouldUsePublicUrlForExternalDevfileRegistryWhenInternalNetworkEnabled(t *testing.T) {
-	os.Setenv("OPENSHIFT_VERSION", "3")
-
-	type testCase struct {
-		name              string
-		cheCR *orgv1.CheCluster
-		expectedDevfileRegistryInternalURL string
+	_, err = r.Reconcile(req)
+	if err != nil {
+		t.Fatalf("Error reconciling: %v", err)
 	}
 
-	// Set the logger to development mode for verbose logs.
-	logf.SetLogger(logf.ZapLogger(true))
-
-	testCases := []testCase{
-		{
-			name: "Should use 'external' devfile registry url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-						ExternalDevfileRegistry: true,
-						DevfileRegistryUrl: "http://external-devfile-registry",
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedDevfileRegistryInternalURL: "http://external-devfile-registry",
-		},
-		{
-			name: "Should use 'external' public devfile registry url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-						ExternalDevfileRegistry: true,
-						DevfileRegistryUrl: "http://external-devfile-registry",
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedDevfileRegistryInternalURL: "http://external-devfile-registry",
-		},
-		{
-			name: "Should use public devfile registry url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-						ExternalDevfileRegistry: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedDevfileRegistryInternalURL: "http://devfile-registry",
-		},
-		{
-			name: "Should use internal devfile registry url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-						ExternalDevfileRegistry: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedDevfileRegistryInternalURL: "http://devfile-registry.eclipse-che.svc:8080",
-		},
+	cheCluster := &orgv1.CheCluster{}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: name, Namespace: namespace}, cheCluster); err != nil {
+		t.Errorf("Unable to get checluster")
 	}
-
-	for _, testCase := range testCases {
-		t.Run(testCase.name, func(t *testing.T) {
-			scheme := scheme.Scheme
-			orgv1.SchemeBuilder.AddToScheme(scheme)
-			scheme.AddKnownTypes(routev1.SchemeGroupVersion, &routev1.Route{})
-
-			cli := fake.NewFakeClientWithScheme(scheme, testCase.cheCR, )
-			nonCachedClient := fake.NewFakeClientWithScheme(scheme, testCase.cheCR)
-			clientSet := fakeclientset.NewSimpleClientset()
-			fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
-			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
-			if !ok {
-				t.Fatal("Error creating fake discovery client")
-			}
-			r := &ReconcileChe{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				scheme:          scheme,
-				tests: true,
-			}
-			req := reconcile.Request{
-				NamespacedName: types.NamespacedName{
-					Name:      name,
-					Namespace: namespace,
-				},
-			}
-
-			clusterAPI := deploy.ClusterAPI{
-				Client: r.client,
-				Scheme: r.scheme,
-			}
-			deployContext := &deploy.DeployContext{
-				CheCluster: testCase.cheCR,
-				ClusterAPI: clusterAPI,
-			}
-
-			// Set up che host for route
-			cheRoute, _ := deploy.GetSpecRoute(deployContext, deploy.DefaultCheFlavor(testCase.cheCR), "che-host", "che-host", 8080, "")
-			r.client.Create(context.TODO(), cheRoute);
-			// Set up keycloak host for route
-			keycloakRoute, _ := deploy.GetSpecRoute(deployContext, "keycloak", "keycloak", "keycloak", 8080, "")
-			r.client.Create(context.TODO(), keycloakRoute)
-			// Set up plugin registry host for route
-			pluginRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "plugin-registry", "plugin-registry", "plugin-registry", 8080, "")
-			r.client.Create(context.TODO(), pluginRegistryRoute)
-			// Set up devfile registry host for route
-			devfileRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "devfile-registry", "devfile-registry", "devfile-registry", 8080, "")
-			r.client.Create(context.TODO(), devfileRegistryRoute)
-
-			_, err := r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-			_, err = r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-
-			cheCm := &corev1.ConfigMap{}
-			if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "che", Namespace: testCase.cheCR.Namespace}, cheCm); err != nil {
-				t.Errorf("ConfigMap %s not found: %s", cheCm.Name, err)
-			}
-
-			devfileRegistryInternalURLActual := cheCm.Data["CHE_WORKSPACE_DEVFILE__REGISTRY__INTERNAL__URL"]
-			devfileRegistryInternalURLExpected := testCase.expectedDevfileRegistryInternalURL
-			if devfileRegistryInternalURLActual != devfileRegistryInternalURLExpected {
-				t.Fatalf("devfile registry internal url must be %s", devfileRegistryInternalURLExpected)
-			}
-		})
-	}
-}
-
-func TestShouldUseCorrectUrlForInternalCheServerURLWhenInternalNetworkEnabled(t *testing.T) {
-	os.Setenv("OPENSHIFT_VERSION", "3")
-
-	type testCase struct {
-		name              string
-		cheCR *orgv1.CheCluster
-		expectedCheServerInternalURL string
-	}
-
-	// Set the logger to development mode for verbose logs.
-	logf.SetLogger(logf.ZapLogger(true))
-
-	testCases := []testCase{
-		{
-			name: "Should use public che-server url, when internal network is disabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedCheServerInternalURL: "http://che-host/api",
-		},
-		{
-			name: "Should use internal che-server url, when internal network is enabled",
-			cheCR: &orgv1.CheCluster{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "CheCluster",
-					APIVersion: "org.eclipse.che/v1",
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      name,
-					Namespace: namespace,
-					ResourceVersion: "1",
-				},
-				Spec: orgv1.CheClusterSpec{
-					Server: orgv1.CheClusterSpecServer{
-						UseInternalClusterSVCNames: true,
-						ExternalDevfileRegistry: false,
-					},
-					Auth: orgv1.CheClusterSpecAuth{
-						OpenShiftoAuth: util.NewBoolPointer(false),
-					},
-				},
-			},
-			expectedCheServerInternalURL: "http://che-host.eclipse-che.svc:8080/api",
-		},
-	}
-
-	for _, testCase := range testCases {
-		t.Run(testCase.name, func(t *testing.T) {
-			scheme := scheme.Scheme
-			orgv1.SchemeBuilder.AddToScheme(scheme)
-			scheme.AddKnownTypes(routev1.SchemeGroupVersion, &routev1.Route{})
-
-			cli := fake.NewFakeClientWithScheme(scheme, testCase.cheCR, )
-			nonCachedClient := fake.NewFakeClientWithScheme(scheme, testCase.cheCR)
-			clientSet := fakeclientset.NewSimpleClientset()
-			fakeDiscovery, ok := clientSet.Discovery().(*fakeDiscovery.FakeDiscovery)
-			fakeDiscovery.Fake.Resources = []*metav1.APIResourceList{}
-			if !ok {
-				t.Fatal("Error creating fake discovery client")
-			}
-			r := &ReconcileChe{
-				client:          cli,
-				nonCachedClient: nonCachedClient,
-				discoveryClient: fakeDiscovery,
-				scheme:          scheme,
-				tests: true,
-			}
-			req := reconcile.Request{
-				NamespacedName: types.NamespacedName{
-					Name:      name,
-					Namespace: namespace,
-				},
-			}
-
-			clusterAPI := deploy.ClusterAPI{
-				Client: r.client,
-				Scheme: r.scheme,
-			}
-			deployContext := &deploy.DeployContext{
-				CheCluster: testCase.cheCR,
-				ClusterAPI: clusterAPI,
-			}
-
-			// Set up che host for route
-			cheRoute, _ := deploy.GetSpecRoute(deployContext, deploy.DefaultCheFlavor(testCase.cheCR), "che-host", "che-host", 8080, "")
-			r.client.Create(context.TODO(), cheRoute);
-			// Set up keycloak host for route
-			keycloakRoute, _ := deploy.GetSpecRoute(deployContext, "keycloak", "keycloak", "keycloak", 8080, "")
-			r.client.Create(context.TODO(), keycloakRoute)
-			// Set up plugin registry host for route
-			pluginRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "plugin-registry", "plugin-registry", "plugin-registry", 8080, "")
-			r.client.Create(context.TODO(), pluginRegistryRoute)
-			// Set up devfile registry host for route
-			devfileRegistryRoute, _ := deploy.GetSpecRoute(deployContext, "plugin-registry", "devfile-registry", "plugin-registry", 8080, "")
-			r.client.Create(context.TODO(), devfileRegistryRoute)
-			
-			_, err := r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-			_, err = r.Reconcile(req)
-			if err != nil {
-				t.Fatalf("Error reconciling: %v", err)
-			}
-
-			cheCm := &corev1.ConfigMap{}
-			if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "che", Namespace: testCase.cheCR.Namespace}, cheCm); err != nil {
-				t.Errorf("ConfigMap %s not found: %s", cheCm.Name, err)
-			}
-
-			cheServerInternalURLActual := cheCm.Data["CHE_API_INTERNAL"]
-			cheServerInternalURLExpected := testCase.expectedCheServerInternalURL
-			if cheServerInternalURLActual != cheServerInternalURLExpected {
-				t.Fatalf("che-server internal url must be %s", cheServerInternalURLExpected)
-			}
-		})
+	if cheCluster.Spec.Server.WorkspaceNamespaceDefault != namespace {
+		t.Error("Failed fallback workspaceNamespaceDefault to execute workspaces in the same namespace with Che")
 	}
 }
 
@@ -1655,7 +1231,7 @@ func createAPIObjects() ([]runtime.Object, discovery.DiscoveryInterface, runtime
 			Name:      "fake-pg-pod",
 			Namespace: "eclipse-che",
 			Labels: map[string]string{
-				"component": "postgres",
+				"component": deploy.PostgresName,
 			},
 		},
 	}
@@ -1695,7 +1271,7 @@ func createAPIObjects() ([]runtime.Object, discovery.DiscoveryInterface, runtime
 	cli := fakeclientset.NewSimpleClientset()
 	fakeDiscovery, ok := cli.Discovery().(*fakeDiscovery.FakeDiscovery)
 	if !ok {
-		fmt.Errorf("Error creating fake discovery client")
+		logrus.Error("Error creating fake discovery client")
 		os.Exit(1)
 	}
 
@@ -1714,6 +1290,9 @@ func InitCheWithSimpleCR() *orgv1.CheCluster {
 			Server: orgv1.CheClusterSpecServer{
 				CheWorkspaceClusterRole: "cluster-admin",
 			},
+			Auth: orgv1.CheClusterSpecAuth{
+				OpenShiftoAuth: util.NewBoolPointer(false),
+			},
 		},
 	}
 }
