@@ -35,7 +35,7 @@ func (r *ReconcileChe) ReconcileFinalizer(instance *orgv1.CheCluster) (err error
 			oAuthClient, err := r.GetOAuthClient(oAuthClientName)
 			if err == nil {
 				if err := r.client.Delete(context.TODO(), oAuthClient); err != nil {
-					logrus.Errorf("Failed to delete %s oAuthClient: %s", oAuthClientName, err)
+					logrus.Errorf("Failed to delete %s oAuthClient: %s", oAuthClientName, err.Error())
 					return err
 				}
 			} else if !errors.IsNotFound(err) {
@@ -43,7 +43,6 @@ func (r *ReconcileChe) ReconcileFinalizer(instance *orgv1.CheCluster) (err error
 				return err
 			}
 			instance.ObjectMeta.Finalizers = util.DoRemoveString(instance.ObjectMeta.Finalizers, oAuthFinalizerName)
-			logrus.Infof("Updating %s CR", instance.Name)
 
 			if err := r.client.Update(context.Background(), instance); err != nil {
 				logrus.Errorf("Failed to update %s CR: %s", instance.Name, err)
@@ -55,7 +54,7 @@ func (r *ReconcileChe) ReconcileFinalizer(instance *orgv1.CheCluster) (err error
 	return nil
 }
 
-func (r *ReconcileChe) DeleteFinalizer(instance *orgv1.CheCluster) (err error) {
+func (r *ReconcileChe) DeleteOAuthFinalizer(instance *orgv1.CheCluster) (err error) {
 	instance.ObjectMeta.Finalizers = util.DoRemoveString(instance.ObjectMeta.Finalizers, oAuthFinalizerName)
 	logrus.Infof("Removing OAuth finalizer on %s CR", instance.Name)
 	if err := r.client.Update(context.Background(), instance); err != nil {
