@@ -35,7 +35,7 @@ func SyncDevfileRegistryToCluster(deployContext *deploy.DeployContext, cheHost s
 	devfileRegistryURL := deployContext.CheCluster.Spec.Server.DevfileRegistryUrl
 	if !deployContext.CheCluster.Spec.Server.ExternalDevfileRegistry {
 		additionalLabels := (map[bool]string{true: deployContext.CheCluster.Spec.Server.DevfileRegistryRoute.Labels, false: deployContext.CheCluster.Spec.Server.DevfileRegistryIngress.Labels})[util.IsOpenShift]
-		endpoint, done, err := expose.Expose(deployContext, cheHost, deploy.DevfileRegistry, additionalLabels)
+		endpoint, done, err := expose.Expose(deployContext, cheHost, deploy.DevfileRegistryName, additionalLabels, deploy.DevfileRegistryName)
 		if !done {
 			return false, err
 		}
@@ -49,7 +49,7 @@ func SyncDevfileRegistryToCluster(deployContext *deploy.DeployContext, cheHost s
 		}
 
 		configMapData := getDevfileRegistryConfigMapData(deployContext.CheCluster, devfileRegistryURL)
-		configMapSpec, err := deploy.GetSpecConfigMap(deployContext, deploy.DevfileRegistry, configMapData)
+		configMapSpec, err := deploy.GetSpecConfigMap(deployContext, deploy.DevfileRegistryName, configMapData, deploy.DevfileRegistryName)
 		if err != nil {
 			return false, err
 		}
@@ -60,11 +60,10 @@ func SyncDevfileRegistryToCluster(deployContext *deploy.DeployContext, cheHost s
 		}
 
 		// Create a new registry service
-		registryLabels := deploy.GetLabels(deployContext.CheCluster, deploy.DevfileRegistry)
-		serviceStatus := deploy.SyncServiceToCluster(deployContext, deploy.DevfileRegistry, []string{"http"}, []int32{8080}, registryLabels)
+		serviceStatus := deploy.SyncServiceToCluster(deployContext, deploy.DevfileRegistryName, []string{"http"}, []int32{8080}, deploy.DevfileRegistryName)
 		if !util.IsTestMode() {
 			if !serviceStatus.Continue {
-				logrus.Info("Waiting on service '" + deploy.DevfileRegistry + "' to be ready")
+				logrus.Info("Waiting on service '" + deploy.DevfileRegistryName + "' to be ready")
 				if serviceStatus.Err != nil {
 					logrus.Error(serviceStatus.Err)
 				}
@@ -73,18 +72,17 @@ func SyncDevfileRegistryToCluster(deployContext *deploy.DeployContext, cheHost s
 			}
 		}
 
-		deployContext.InternalService.DevfileRegistryHost = fmt.Sprintf("http://%s.%s.svc:8080", deploy.DevfileRegistry, deployContext.CheCluster.Namespace)
+		deployContext.InternalService.DevfileRegistryHost = fmt.Sprintf("http://%s.%s.svc:8080", deploy.DevfileRegistryName, deployContext.CheCluster.Namespace)
 
 		// Deploy devfile registry
-		deploymentStatus := SyncDevfileRegistryDeploymentToCluster(deployContext)
+		provisioned, err := SyncDevfileRegistryDeploymentToCluster(deployContext)
 		if !util.IsTestMode() {
-			if !deploymentStatus.Continue {
-				logrus.Info("Waiting on deployment '" + deploy.DevfileRegistry + "' to be ready")
-				if deploymentStatus.Err != nil {
-					logrus.Error(deploymentStatus.Err)
+			if !provisioned {
+				logrus.Info("Waiting on deployment '" + deploy.DevfileRegistryName + "' to be ready")
+				if err != nil {
+					logrus.Error(err)
 				}
-
-				return false, deploymentStatus.Err
+				return provisioned, err
 			}
 		}
 	}
