@@ -91,7 +91,7 @@ func SyncCheConfigMapToCluster(deployContext *deploy.DeployContext) (*corev1.Con
 	if err != nil {
 		return nil, err
 	}
-	specConfigMap, err := deploy.GetSpecConfigMap(deployContext, CheConfigMapName, data)
+	specConfigMap, err := deploy.GetSpecConfigMap(deployContext, CheConfigMapName, data, deploy.DefaultCheFlavor(deployContext.CheCluster))
 	if err != nil {
 		return nil, err
 	}
@@ -104,28 +104,19 @@ func SyncCheConfigMapToCluster(deployContext *deploy.DeployContext) (*corev1.Con
 func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string]string, err error) {
 	cheHost := deployContext.CheCluster.Spec.Server.CheHost
 	keycloakURL := deployContext.CheCluster.Spec.Auth.IdentityProviderURL
-	isOpenShift, isOpenshift4, err := util.DetectOpenShift()
-	if err != nil {
-		logrus.Errorf("Failed to get current infra: %s", err)
-	}
 	cheFlavor := deploy.DefaultCheFlavor(deployContext.CheCluster)
 	infra := "kubernetes"
-	if isOpenShift {
+	if util.IsOpenShift {
 		infra = "openshift"
 	}
 	tls := "false"
 	openShiftIdentityProviderId := "NULL"
-	defaultTargetNamespaceDefault := deployContext.CheCluster.Namespace // By default Che SA has right in the namespace where Che in installed ...
-	if isOpenShift && util.IsOAuthEnabled(deployContext.CheCluster) {
-		// ... But if the workspace is created under the openshift identity of the end-user,
-		// Then we'll have rights to create any new namespace
-		defaultTargetNamespaceDefault = "<username>-" + cheFlavor
+	if util.IsOpenShift && util.IsOAuthEnabled(deployContext.CheCluster) {
 		openShiftIdentityProviderId = "openshift-v3"
-		if isOpenshift4 {
+		if util.IsOpenShift4 {
 			openShiftIdentityProviderId = "openshift-v4"
 		}
 	}
-	defaultTargetNamespace := util.GetValue(deployContext.CheCluster.Spec.Server.WorkspaceNamespaceDefault, defaultTargetNamespaceDefault)
 	namespaceAllowUserDefined := strconv.FormatBool(deployContext.CheCluster.Spec.Server.AllowUserDefinedWorkspaceNamespaces)
 	tlsSupport := deployContext.CheCluster.Spec.Server.TlsSupport
 	protocol := "http"
@@ -180,6 +171,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 	cheMultiUser := deploy.GetCheMultiUser(deployContext.CheCluster)
 	workspaceExposure := deploy.GetSingleHostExposureType(deployContext.CheCluster)
 	singleHostGatewayConfigMapLabels := labels.FormatLabels(util.GetMapValue(deployContext.CheCluster.Spec.Server.SingleHostGatewayConfigMapLabels, deploy.DefaultSingleHostGatewayConfigMapLabels))
+	workspaceNamespaceDefault := util.GetWorkspaceNamespaceDefault(deployContext.CheCluster)
 
 	cheAPI := protocol + "://" + cheHost + "/api"
 	var keycloakInternalURL, pluginRegistryInternalURL, devfileRegistryInternalURL, cheInternalAPI string
@@ -219,7 +211,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 		CheDebugServer:                         cheDebug,
 		CheInfrastructureActive:                infra,
 		CheInfraKubernetesServiceAccountName:   "che-workspace",
-		DefaultTargetNamespace:                 defaultTargetNamespace,
+		DefaultTargetNamespace:                 workspaceNamespaceDefault,
 		NamespaceAllowUserDefined:              namespaceAllowUserDefined,
 		PvcStrategy:                            pvcStrategy,
 		PvcClaimSize:                           pvcClaimSize,
@@ -272,7 +264,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 	err = json.Unmarshal(out, &cheEnv)
 
 	// k8s specific envs
-	if !isOpenShift {
+	if !util.IsOpenShift {
 		k8sCheEnv := map[string]string{
 			"CHE_INFRA_KUBERNETES_POD_SECURITY__CONTEXT_FS__GROUP":     securityContextFsGroup,
 			"CHE_INFRA_KUBERNETES_POD_SECURITY__CONTEXT_RUN__AS__USER": securityContextRunAsUser,
@@ -284,10 +276,7 @@ func GetCheConfigMapData(deployContext *deploy.DeployContext) (cheEnv map[string
 
 		// Add TLS key and server certificate to properties when user workspaces should be created in another
 		// than Che server namespace, from where the Che TLS secret is not accessable
-		k8sDefaultNamespace := deployContext.CheCluster.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"]
-		if (defaultTargetNamespace != "" && defaultTargetNamespace != deployContext.CheCluster.Namespace) ||
-			(k8sDefaultNamespace != "" && k8sDefaultNamespace != deployContext.CheCluster.Namespace) {
-
+		if !util.IsWorkspaceInSameNamespaceWithChe(deployContext.CheCluster) {
 			cheTLSSecret, err := deploy.GetClusterSecret(deployContext.CheCluster.Spec.K8s.TlsSecretName, deployContext.CheCluster.ObjectMeta.Namespace, deployContext.ClusterAPI)
 			if err != nil {
 				return nil, err
