@@ -32,6 +32,7 @@ import (
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/discovery"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
@@ -346,10 +347,38 @@ func NewBoolPointer(value bool) *bool {
 	return &variable
 }
 
-// IsOAuthEnabled return true when oAuth is enable for CheCluster resource, otherwise false.
+// IsOAuthEnabled returns true when oAuth is enable for CheCluster resource, otherwise false.
 func IsOAuthEnabled(c *orgv1.CheCluster) bool {
 	if c.Spec.Auth.OpenShiftoAuth != nil && *c.Spec.Auth.OpenShiftoAuth {
 		return true
 	}
 	return false
 }
+
+// IsWorkspaceInSameNamespaceWithChe return true when Che workspaces will be executed in the same namespace with Che, otherwise returns false.
+func IsWorkspaceInSameNamespaceWithChe(cr *orgv1.CheCluster) bool {
+	return GetWorkspaceNamespaceDefault(cr) == cr.Namespace
+}
+
+// GetWorkspaceNamespaceDefault - returns workspace namespace default strategy, which points on the namespaces used for workspaces execution.
+func GetWorkspaceNamespaceDefault(cr *orgv1.CheCluster) string {
+	if cr.Spec.Server.CustomCheProperties != nil {
+		k8sNamespaceDefault := cr.Spec.Server.CustomCheProperties["CHE_INFRA_KUBERNETES_NAMESPACE_DEFAULT"]
+		if k8sNamespaceDefault != "" {
+			return k8sNamespaceDefault
+		}
+	}
+
+	workspaceNamespaceDefault := cr.Namespace
+	if IsOpenShift && IsOAuthEnabled(cr) {
+		workspaceNamespaceDefault = "<username>-" + cr.Spec.Server.CheFlavor
+	}
+	return GetValue(cr.Spec.Server.WorkspaceNamespaceDefault, workspaceNamespaceDefault)
+}
+
+func GetResourceQuantity(value string, defaultValue string) resource.Quantity {
+	if value != "" {
+		return resource.MustParse(value)
+	}
+	return resource.MustParse(defaultValue)
+}
